import functools
import json
import pathlib

from matplotlib.cm import register_cmap
from matplotlib.colors import ListedColormap

mod_dir = pathlib.Path(__file__).parent.parent


@functools.lru_cache
def colormaps():
    '''return a dictionary of colormaps'''

    with open(mod_dir / 'data' / 'colormaps.json', mode='r') as f:
        data = json.load(f)

    cmaps = {}
    for k, v in data.items():
        key = k.replace('-', '_')
        cmaps[key] = ListedColormap(v, name=key)

        key_r = key + '_r'
        cmaps[key_r] = cmaps[key].reversed()

    return cmaps


# register colormaps in this namespace
_cmaps = colormaps()

lcs = locals()
lcs.update(**_cmaps)

# register colormaps in the matplotlib namespace
for k, v in _cmaps.items():
    register_cmap(name=v.name, cmap=v)



dark = {
    'text': '#ebebec',
    'background': '#1b1e23',
    'primary': '#e4e4e4',
    'secondary': '#808080',
    'muted': '#363a3e',
    'red': '#f07071',
    'orange': '#ea9755',
    'yellow': '#d4c05e',
    'green': '#7eb36a',
    'teal': '#64b9c4',
    'blue': '#85a2f7',
    'purple': '#bc85d9',
    'pink': '#e587b6',
    'grey': '#a9b4c4',
}

light = {
    'text': '#1b1e23',
    'background': '#FFFFFF',
    'primary': '#1b1e23',
    'secondary': '#808080',
    'muted': '#b0afb1',
    'red': '#f07071',
    'orange': '#ea9755',
    'yellow': '#d4c05e',
    'green': '#7eb36a',
    'teal': '#64b9c4',
    'blue': '#85a2f7',
    'purple': '#bc85d9',
    'pink': '#e587b6',
    'grey': '#a9b4c4',
}


def colors(mode='dark'):
    if mode in ['dark', 'carbonplan_dark']:
        return dark
    elif mode in ['light', 'carbonplan_light']:
        return light
    else:
        raise ValueError('unknown color mode %s' % mode)


def hex_to_rgb(hex):
    """
    Converts hex to rgb colours

    Parameters
    ----------
    hex : string
        6 characters representing a hex color

    Returns
    -------
    rgb : tuple (length 3)
        RGB values

    References
    ----------
    https://towardsdatascience.com/beautiful-custom-colormaps-with-matplotlib-5bab3d1f0e72
    """
    hex = hex.strip('#')  # removes hash symbol if present
    lv = len(hex)

    return tuple(int(hex[i : i + lv // 3], 16) for i in range(0, lv, lv // 3))


def rgb_to_dec(value):
    """
    Converts rgb to decimal colours (i.e. divides each value by 256)

    Parameters
    ----------
    value: tuple (length 3)
        RGB values

    Returns
    -------
    dec : tuple (length 3)
        Decimal color values

    References
    ----------
    https://towardsdatascience.com/beautiful-custom-colormaps-with-matplotlib-5bab3d1f0e72
    """
    return tuple(v / 256 for v in value)



font = 'relative-mono-11-pitch-pro, Menlo, monospace'
labelfont = 'relative-mono-11-pitch-pro, Menlo, monospace'
sourcefont = 'relative-mono-11-pitch-pro, Menlo, monospace'


def theme(colors):
    markColor = colors['text']
    axisColor = colors['text']
    gridColor = colors['grey']
    backgroundColor = colors['background']
    fontWeight = 400
    symbolSize = 200

    axis = {
        'domainColor': gridColor,
        'gridColor': gridColor,
        'tickColor': gridColor,
        'labelFont': labelfont,
        'titleFont': font,
        'labelColor': axisColor,
        'titleColor': axisColor,
    }

    return {
        'config': {
            'arc': {'fill': markColor},
            'area': {'fill': markColor},
            'axisX': {'grid': False},
            'axisY': {'grid': False},
            'axisBottom': axis,
            'axisLeft': axis,
            'view': {'stroke': None},
            'background': backgroundColor,
            'group': {'fill': backgroundColor},
            'legend': {
                'labelFont': labelfont,
                'symbolSize': symbolSize,
                'symbolType': 'circle',
                'titleFont': font,
            },
            'line': {'color': markColor, 'stroke': markColor},
            'trail': {'color': markColor, 'stroke': markColor},
            'path': {'stroke': markColor},
            'point': {'filled': True, 'color': markColor, 'size': symbolSize, 'cursor': 'pointer'},
            'range': {
                'category': [
                    colors[c]
                    for c in [
                        'blue',
                        'orange',
                        'green',
                        'red',
                        'purple',
                        'pink',
                        'grey',
                        'yellow',
                        'teal',
                    ]
                ],
                # TODO.
                #    "diverging": [
                #        "#dc0d12",
                #        "#e9686b",
                #        "#fbe1e1",
                #        "#dff4f9",
                #        "#81d1e6",
                #        "#03a3cd"
                #    ],
                #    "heatmap": [
                #        "#fcdfef",
                #        "#f8bfde",
                #        "#f59fce",
                #        "#f180be",
                #        "#ee60ad",
                #        "#eb409d",
                #        "#e7208c",
                #        "#e4007c",
                #    ],
            },
            'symbol': {'shape': 'circle'},
            'style': {
                'bar': {'fill': markColor},
                'text': {'font': sourcefont, 'fontWeight': fontWeight},
            },
            'title': {'anchor': 'start', 'fontWeight': fontWeight, 'font': font},
            'header': {'fontWeight': fontWeight, 'labelFont': labelfont, 'titleFont': font},
        },
    }


def dark():
    '''CarbonPlan dark theme entrypoint'''
    from .colors import dark

    return theme(dark)


def light():
    '''CarbonPlan light theme entrypoint'''
    from .colors import light

    return theme(light)



import itertools

import pytest
from matplotlib import cm
from matplotlib.colors import ListedColormap

from carbonplan_styles.mpl import colormaps, set_theme


@pytest.mark.parametrize('style', ['carbonplan_dark', 'carbonplan_light'])
def test_set_style(style):
    set_theme(style=style)


def test_colormaps():
    base_names = [
        'reds',
        'oranges',
        'yellows',
        'greens',
        'teals',
        'blues',
        'purples',
        'pinks',
        'greys',
        'fire',
        'earth',
        'water',
        'heart',
        'wind',
        'warm',
        'cool',
        'pinkgreen',
        'redteal',
        'orangeblue',
        'yellowpurple',
        'redgrey',
        'orangegrey',
        'yellowgrey',
        'greengrey',
        'tealgrey',
        'bluegrey',
        'purplegrey',
        'pinkgrey',
        'rainbow',
        'sinebow',
    ]

    explicit_cmaps = colormaps.colormaps()

    for pieces in itertools.product(base_names, ['_light', '_dark'], '', '_r'):
        cmap_name = ''.join(pieces)

        exp_cmap = explicit_cmaps[cmap_name]
        assert isinstance(exp_cmap, ListedColormap)

        imp_cmap = getattr(colormaps, cmap_name)
        assert isinstance(imp_cmap, ListedColormap)

        mpl_cmap = cm.get_cmap(cmap_name)
        assert isinstance(mpl_cmap, ListedColormap)

        assert exp_cmap.colors == imp_cmap.colors
        assert exp_cmap.name == imp_cmap.name
        assert exp_cmap.colors == mpl_cmap.colors
        assert exp_cmap.name == mpl_cmap.name



import warnings

import matplotlib as mpl
import matplotlib.colors as mcolors
import numpy as np
import seaborn as sns

from ..colors import colors, hex_to_rgb, rgb_to_dec
from . import colormaps  # noqa

palette_colors = ['blue', 'orange', 'green', 'red', 'purple', 'pink', 'grey', 'yellow', 'teal']


def get_style_config(mode):

    c = colors(mode=mode)

    if mode == 'light':
        background = 'white'  # instead of c['background']
    else:
        background = c['background']

    style = {
        'axes.facecolor': background,
        'axes.edgecolor': c['secondary'],
        'axes.grid': False,
        'axes.axisbelow': True,
        'axes.labelcolor': c['text'],
        'figure.facecolor': background,
        'grid.color': c['secondary'],
        'grid.linestyle': '-',
        'text.color': c['text'],
        'xtick.color': c['secondary'],
        'ytick.color': c['secondary'],
        'xtick.labelcolor': c['blue'],
        'ytick.labelcolor': c['blue'],
        'xtick.direction': 'out',
        'ytick.direction': 'out',
        'lines.markeredgewidth': 0.0,
        'lines.solid_capstyle': 'round',
        'patch.edgecolor': c['secondary'],
        'patch.force_edgecolor': True,
        'image.cmap': 'cool_light',
        'font.weight': '400',
        'font.family': 'monospace',
        'font.monospace': ['relative-mono-11-pitch-pro', 'Menlo', 'monospace'],
        'font.sans-serif': [
            'relative-book-pro',
            'Roboto',
            'system-ui',
            '-apple-system',
            'BlinkMacSystemFont',
        ],
        'xtick.bottom': True,
        'xtick.top': False,
        'ytick.left': True,
        'ytick.right': False,
        'axes.spines.left': True,
        'axes.spines.bottom': True,
        'axes.spines.right': False,
        'axes.spines.top': False,
        'scatter.marker': 'o',
        'scatter.edgecolors': None,
    }

    return style, c


def set_theme(
    context='notebook', style='carbonplan_light', font='monospace', font_scale=1, rc=None
):
    """Set multiple theme parameters in one step.

    Each set of parameters can be set directly or temporarily, see the
    referenced functions below for more information.

    Parameters
    ----------
    context : string or dict
        Plotting context parameters, see :func:`plotting_context`.
    style : string or dict
        Axes style parameters, see :func:`axes_style`.
    font : string
        Font family, see matplotlib font manager.
    rc : dict or None
        Dictionary of rc parameter mappings to override the above.

    See Also
    --------
    seaborn.set_theme
    """
    # set context
    sns.set_context(context, font_scale)

    # set style
    style_dict, colors_dict = get_style_config(style)
    sns.set_style(style_dict, rc={'font.family': font})

    # set color palette
    c = [colors_dict[k] for k in palette_colors]
    palette = sns.color_palette(c)
    sns.set_palette(palette)

    if rc is not None:
        mpl.rcParams.update(rc)


def get_continuous_cmap(hex_list, float_list=None, name=None):
    """
    Creates and returns a color map that can be used in heat map figures.
    If float_list is not provided, colour map graduates linearly between each color in hex_list.
    If float_list is provided, each color in hex_list is mapped to the respective location in float_list.

    Parameters
    ----------
    hex_list : list
        Hex code strings
    float_list: list, optional
        List of floats between 0 and 1, same length as hex_list. Must start with 0 and end with 1.

    Returns
    ----------
    cmap : mcolors.LinearSegmentedColormap

    References
    ----------
    https://towardsdatascience.com/beautiful-custom-colormaps-with-matplotlib-5bab3d1f0e72
    """

    warnings.warn(
        'get_continuous_cmap is deprecated, use carbonplan_styles.mpl.colormaps instead',
        DeprecationWarning,
    )

    if name is None:
        name = '-'.join(hex_list)

    rgb_list = [rgb_to_dec(hex_to_rgb(i)) for i in hex_list]

    if float_list:
        pass
    else:
        float_list = list(np.linspace(0, 1, len(rgb_list)))

    cdict = dict()
    for num, col in enumerate(['red', 'green', 'blue']):
        cdict[col] = [
            [float_list[i], rgb_list[i][num], rgb_list[i][num]] for i in range(len(float_list))
        ]
    cmap = mcolors.LinearSegmentedColormap(name, segmentdata=cdict, N=256)

    return cmap


def get_colormap(name):

    warnings.warn(
        'get_colormap is deprecated, use carbonplan_styles.mpl.colormaps instead',
        DeprecationWarning,
    )

    if name == 'blues':
        return get_continuous_cmap(['#CFE0F9', '#588EF9', '#0432A5'])
    elif name == 'pinks':
        return get_continuous_cmap(['#F9C7ED', '#E563BA', '#770361'])
    elif name == 'reds':
        return get_continuous_cmap(['#F9D3BD', '#E87A3D', '#752003'])



import altair as alt
import pytest


@pytest.mark.parametrize('theme', ['carbonplan_dark', 'carbonplan_light'])
def test_enable_theme(theme):
    assert theme in alt.themes.names()
    assert alt.themes.enable(theme)



from pkg_resources import DistributionNotFound, get_distribution

from . import altair, colors, mpl  # noqa

try:
    version = get_distribution(__name__).version
except DistributionNotFound:  # pragma: no cover
    version = '0.0.0'  # pragma: no cover
__version__ = version


banner = """-------------------------------
   ███        ███        ███
  ██          ███          ██
 ██           ███           ██
 ██      ██   ███   ██      ██
 ██       ███ ███ ███       ██
  ██         █████         ██
   ███        ███        ███

/ (carbon)plan
/ carbonplan.org
/ https://github.com/carbonplan
-------------------------------
"""



import os

from setuptools import find_packages, setup

with open('requirements.txt') as f:
    install_requires = f.read().strip().split('\n')


if os.path.exists('README.md'):
    with open('README.md') as f:
        long_description = f.read()
else:
    long_description = ''


CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Intended Audience :: Science/Research',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Topic :: Scientific/Engineering',
]

ENTRY_POINTS = {
    'altair.vegalite.v4.theme': [
        'carbonplan_dark = carbonplan_styles.altair:dark',
        'carbonplan_light = carbonplan_styles.altair:light',
    ],
}


setup(
    name='carbonplan-styles',
    description='CarbonPlan plotting styles',
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires='>=3.7',
    maintainer='Joe Hamman',
    maintainer_email='joe@carbonplan.org',
    url='https://github.com/carbonplan/styles',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    entry_points=ENTRY_POINTS,
    install_requires=install_requires,
    keywords=['matplotlib', 'altair'],
    classifiers=CLASSIFIERS,
    use_scm_version={'version_scheme': 'post-release', 'local_scheme': 'dirty-tag'},
    setup_requires=['setuptools_scm', 'setuptools>=30.3.0'],
)
